SimCompare = LibStub("AceAddon-3.0"):NewAddon("SimCompare", "AceConsole-3.0", "AceEvent-3.0")

local OFFSET_ITEM_ID = 1
local OFFSET_ENCHANT_ID = 2
local OFFSET_GEM_ID_1 = 3
local OFFSET_GEM_ID_2 = 4
local OFFSET_GEM_ID_3 = 5
local OFFSET_GEM_ID_4 = 6
local OFFSET_GEM_BASE = OFFSET_GEM_ID_1
local OFFSET_SUFFIX_ID = 7
local OFFSET_FLAGS = 11
local OFFSET_CONTEXT = 12
local OFFSET_BONUS_ID = 13
local OFFSET_UPGRADE_ID = 14 -- Flags = 0x4

local AzeriteEmpoweredItem  = _G.C_AzeriteEmpoweredItem

local simcSlotNames = {
  INVTYPE_HEAD = "head",
  INVTYPE_NECK = "neck",
  INVTYPE_SHOULDER = "shoulder",
  INVTYPE_BODY = "shirt",
  INVTYPE_CHEST = "chest",
  INVTYPE_ROBE = "chest",
  INVTYPE_WAIST = "waist",
  INVTYPE_LEGS = "legs",
  INVTYPE_FEET = "feet",
  INVTYPE_WRIST = "wrist",
  INVTYPE_HAND = "hands",
  INVTYPE_FINGER = "finger1",
  INVTYPE_TRINKET = "trinket1",
  INVTYPE_CLOAK = "back",
  INVTYPE_WEAPON = "main_hand",
  INVTYPE_SHIELD = "off_hand",
  INVTYPE_2HWEAPON = "main_hand",
  INVTYPE_WEAPONMAINHAND = "main_hand",
  INVTYPE_WEAPONOFFHAND = "off_hand",
  INVTYPE_HOLDABLE = "off_hand",
  INVTYPE_RANGED = "main_hand",
  INVTYPE_THROWN = "main_hand",
  INVTYPE_RANGEDRIGHT = "main_hand",
  INVTYPE_RELIC = "main_hand",
  INVTYPE_TABARD = "tabard",
}

-- local simcSlotNames = {'head','neck','shoulder','back','chest','shirt','tabard','wrist','hands','waist','legs','feet'
-- ,'finger1','finger2','trinket1','trinket2','main_hand','off_hand','ammo'}

-- "INVTYPE_HEAD"	Head	1
-- "INVTYPE_NECK"	Neck	2
-- "INVTYPE_SHOULDER"	Shoulder	3
-- "INVTYPE_BODY"	Shirt	4
-- "INVTYPE_CHEST"	Chest	5
-- "INVTYPE_ROBE"	Chest	5
-- "INVTYPE_WAIST"	Waist	6
-- "INVTYPE_LEGS"	Legs	7
-- "INVTYPE_FEET"	Feet	8
-- "INVTYPE_WRIST"	Wrist	9
-- "INVTYPE_HAND"	Hands	10
-- "INVTYPE_FINGER"	Fingers	11,12
-- "INVTYPE_TRINKET"	Trinkets	13,14
-- "INVTYPE_CLOAK"	Cloaks	15
-- "INVTYPE_WEAPON"	One-Hand	16,17
-- "INVTYPE_SHIELD"	Shield	17
-- "INVTYPE_2HWEAPON"	Two-Handed	16
-- "INVTYPE_WEAPONMAINHAND"	Main-Hand Weapon	16
-- "INVTYPE_WEAPONOFFHAND"	Off-Hand Weapon	17
-- "INVTYPE_HOLDABLE"	Held In Off-Hand	17
-- "INVTYPE_RANGED"	Bows	18
-- "INVTYPE_THROWN"	Ranged	18
-- "INVTYPE_RANGEDRIGHT"	Wands, Guns, and Crossbows	18
-- "INVTYPE_RELIC"	Relics	18
-- "INVTYPE_TABARD"	Tabard	19

function SimCompare:OnInitialize()
  self:RegisterChatCommand("compare", "Command")
end

function SimCompare:Command(args)
  local playerName = UnitName('player')
  local itemString = itemStringFromItemLink(args)

  if not itemString then
    self:Print("not a comparable item")
    return
  end

  local compareString = "!sim " .. playerName .. " -ci " .. itemString

  SimCompareFrame:Show()
  SimCompareFrameScrollText:Show()
  SimCompareFrameScrollText:SetText(compareString)
  SimCompareFrameScrollText:HighlightText()
  SimCompareFrameScrollText:SetScript("OnEscapePressed", function(self)
    SimCompareFrame:Hide()
  end)
end

function SimCompare:Print(message)
  DEFAULT_CHAT_FRAME:AddMessage(message)
end

-- borrowed from Simulationcraft addon, modified to take an itemLink
function itemStringFromItemLink(itemLink)
  local itemSplit = itemSplit(itemLink)
  local simcItemOptions = {}

  -- Slot
  local _, _, _, _, _, _, _, _, itemSlot = GetItemInfo(itemLink)
  -- SimCompare:Print("itemSlot: " .. itemSlot)

  if not simcSlotNames[itemSlot] then
    return
  end

  -- Item id
  local itemId = itemSplit[OFFSET_ITEM_ID]
  simcItemOptions[#simcItemOptions + 1] = ',id=' .. itemId

  -- Enchant
  if itemSplit[OFFSET_ENCHANT_ID] > 0 then
    simcItemOptions[#simcItemOptions + 1] = 'enchant_id=' .. itemSplit[OFFSET_ENCHANT_ID]
  end

  -- New style item suffix, old suffix style not supported
  if itemSplit[OFFSET_SUFFIX_ID] ~= 0 then
    simcItemOptions[#simcItemOptions + 1] = 'suffix=' .. itemSplit[OFFSET_SUFFIX_ID]
  end

  local flags = itemSplit[OFFSET_FLAGS]

  local bonuses = {}

  for index=1, itemSplit[OFFSET_BONUS_ID] do
    bonuses[#bonuses + 1] = itemSplit[OFFSET_BONUS_ID + index]
  end

  if #bonuses > 0 then
    simcItemOptions[#simcItemOptions + 1] = 'bonus_id=' .. table.concat(bonuses, '/')
  end

  local linkOffset = OFFSET_BONUS_ID + #bonuses + 1

  -- Upgrade level
  if bit.band(flags, 0x4) == 0x4 then
    local upgradeId = itemSplit[linkOffset]
    if upgradeTable and upgradeTable[upgradeId] ~= nil and upgradeTable[upgradeId] > 0 then
      simcItemOptions[#simcItemOptions + 1] = 'upgrade=' .. upgradeTable[upgradeId]
    end
    linkOffset = linkOffset + 1
  end

  -- Some leveling quest items seem to use this, it'll include the drop level of the item
  if bit.band(flags, 0x200) == 0x200 then
    simcItemOptions[#simcItemOptions + 1] = 'drop_level=' .. itemSplit[linkOffset]
    linkOffset = linkOffset + 1
  end

  -- Get item creation context. Can be used to determine unlock/availability of azerite tiers for 3rd parties
  if itemSplit[OFFSET_CONTEXT] ~= 0 then
    simcItemOptions[#simcItemOptions + 1] = 'context=' .. itemSplit[OFFSET_CONTEXT]
  end

  -- Azerite powers - only run in BfA client
  if AzeriteEmpoweredItem then
    -- SimCompare:Print("itemId:"..itemId)
    if AzeriteEmpoweredItem.IsAzeriteEmpoweredItemByID(itemId) then
      simcItemOptions[#simcItemOptions + 1] = 'azerite_powers='
    end
  end

  local itemStr = simcSlotNames[itemSlot] .. "=" .. table.concat(simcItemOptions, ',')

  return itemStr
end

function itemSplit(itemLink)
  local itemString = string.match(itemLink, "item:([%-?%d:]+)")
  local itemSplit = {}

  -- Split data into a table
  for _, v in ipairs({strsplit(":", itemString)}) do
    if v == "" then
      itemSplit[#itemSplit + 1] = 0
    else
      itemSplit[#itemSplit + 1] = tonumber(v)
    end
  end

  return itemSplit
end
